import base64
import copy
import json

from indy.crypto import crypto_verify

from verity_sdk.protocols.Protocol import Protocol
from verity_sdk.transports import send_packed_message
from verity_sdk.utils import _pack_message_for_verity_direct, unpack_message, EVERNYM_MSG_QUALIFIER
from verity_sdk.utils.Context import Context


class Provision(Protocol):
    """
    The Provision protocol is used to provision an agent on the verity-application. This agent is the element that
    represents an self-sovereign Identity (organization or otherwise). The protocol is unique that two ways. First it
    only run once per agent. So it is a one-time operation to start things. If run again, it will create a new and
    independent agent. Second, the interface for this protocol use an incomplete context object. It is during this
    protocol that the context object is fully filled out.
    """

    MSG_FAMILY = 'agent-provisioning'
    """the family name for the message family"""

    MSG_FAMILY_VERSION = '0.7'
    """the version for the message family"""

    # Messages
    CREATE_EDGE_AGENT = 'create-edge-agent'
    """Name for 'create-edge-agent' control message"""

    _token = None

    def __init__(self, token=None):
        """
        Args:
            token (str): an agent creation token that is provided by Evernym that authorize the creation of an agent
        """
        super().__init__(
            self.MSG_FAMILY,
            self.MSG_FAMILY_VERSION,
            msg_qualifier=EVERNYM_MSG_QUALIFIER,
        )
        self._token = token

    # noinspection PyMethodMayBeStatic
    async def _send_to_verity(self, context, packed_msg) -> dict:
        resp_bytes = send_packed_message(context, packed_msg)

        return await unpack_message(context, resp_bytes)

    @staticmethod
    async def validate_token(token: str):
        """
        Checks the token for valid structure and signature. No explicit return but will throw exception if invalid.

        Args:
            token (str): an agent creation token that is validated
        """
        tokenObj = json.loads(token)
        data = (
                tokenObj['nonce'] +
                tokenObj['timestamp'] +
                tokenObj['sponseeId'] +
                tokenObj['sponsorId']
        ).encode('utf-8')

        valid = await crypto_verify(
            tokenObj['sponsorVerKey'],
            data,
            base64.b64decode(tokenObj['sig'])
        )

        if not valid:
            raise Exception('Invalid provision token -- signature does not validate')

    async def provision(self, context: Context) -> Context:
        """
        Sends provisioning message that directs the creation of an agent to the to verity-application

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            An updated Context object with details inserted from the provisioning process
        """
        if self._token is not None:
            await self.validate_token(self._token)

        msg = await self.provision_msg_packed(context)

        resp = await self._send_to_verity(context, msg)
        if resp['@type'] == self.message_type(self.PROBLEM_REPORT):
            raise Exception(resp['msg'])

        domain_did: str = resp['selfDID']
        verity_agent_verkey: str = resp['agentVerKey']

        new_context = copy.copy(context)
        new_context.domain_did = domain_did
        new_context.verity_agent_verkey = verity_agent_verkey
        return new_context

    async def provision_msg_packed(self, context: Context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await _pack_message_for_verity_direct(
            context.wallet_handle,
            self.provision_msg(context),
            context.verity_public_did,
            context.verity_public_verkey,
            context.sdk_verkey,
            context.verity_public_verkey
        )

    def provision_msg(self, context: Context):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.CREATE_EDGE_AGENT)
        msg['requesterVk'] = context.sdk_verkey

        if self._token is not None:
            msg['provisionToken'] = json.loads(self._token)

        return msg



from urllib.parse import urljoin
import requests


def msg_endpoint(url: str) -> str:
    """
    Joins a given url to the message route on the verity-application

    Args:
        url: the given url to be jointed
    Returns:
        str: the full url to the message route
    """
    return urljoin(url, '/agency/msg')


def send_packed_message(context, packed_message: bytes):
    """
    Sends a packaged message to the verity-application.

    Args:
        context (Context): an instance of the Context object initialized to a verity-application agent
        packed_message (bytes): the message to send to Verity
    Returns:
        bytes: bytes returned from the verity-application
    """
    url = msg_endpoint(context.verity_url)
    try:
        r = requests.post(
            url,
            data=packed_message,
            headers={'Content-Type': 'application/octet-stream'}
        )
        r.raise_for_status()
        return r.content
    except requests.RequestException as e:
        print('ERROR: Failed to POST message to {}'.format(url))
        raise e



from verity_sdk.protocols.Protocol import Protocol
from verity_sdk.utils import COMMUNITY_MSG_QUALIFIER
from verity_sdk.utils.Exeptions import WrongSetupException


class PresentProof(Protocol):
    """
    The PresentProof protocol allows one self-sovereign party ask another self-sovereign party for a private
    and verifiable presentation from credentials they hold. This request can be restricted to certain selectable
    restrictions.
    """
    MSG_FAMILY = 'present-proof'
    """the family name for the message family"""
    MSG_FAMILY_VERSION = '1.0'
    """the version for the message family"""

    PROOF_REQUEST = 'request'
    """Name for 'request' control message"""
    STATUS = 'status'
    """Name for 'status' control message"""
    REJECT = 'reject'
    """Name for 'reject' control message"""
    ACCEPT_PROPOSAL = 'accept-proposal'
    """Name for 'accept-proposal' control message"""
    PRESENTATION_RESULT = 'presentation-result'
    """Name for 'presentation-result' signal message"""
    PROTOCOL_INVITATION = 'protocol-invitation'
    """Name for 'protocol-invitation' signal message"""


    def __init__(self,
                 for_relationship: str,
                 thread_id: str = None,
                 name: str = None,
                 proof_attrs=None,
                 proof_predicates=None,
                 by_invitation: bool = False):
        """
        Args:
            for_relationship (str): the relationship identifier (DID) for the pairwise relationship that will be used
            thread_id (str): the thread id of the already started protocol
            name (str): a human readable name for the given request
            proof_attrs (List[Dict]): an array of attribute based restrictions
            proof_predicates (List[Dict]): an array of predicate based restrictions (">=", ">", "<=", "<")
            by_invitation (bool): flag to create out-of-band invitation as a part of the PresentProof protocol
        """
        super().__init__(
            self.MSG_FAMILY,
            self.MSG_FAMILY_VERSION,
            msg_qualifier=COMMUNITY_MSG_QUALIFIER,
            thread_id=thread_id
        )
        self.for_relationship = for_relationship
        self.name = name
        self.proof_attrs = proof_attrs
        self.proof_predicates = proof_predicates
        self.by_invitation = by_invitation
        self.created = thread_id is None

    async def request(self, context):
        """
        Directs verity-application to request a presentation of proof.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.request_msg_packed(context))

    def request_msg(self):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        if not self.created:
            raise WrongSetupException('Unable to request presentation when NOT starting the interaction')
        msg = self._get_base_message(self.PROOF_REQUEST)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        msg['name'] = self.name
        msg['proof_attrs'] = self.proof_attrs
        if self.proof_predicates:
            msg['proof_predicates'] = self.proof_predicates
        msg['by_invitation'] = self.by_invitation
        return msg

    async def request_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.request_msg())

    async def status(self, context):
        """
        Ask for status from the verity-application agent

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.status_msg_packed(context))

    def status_msg(self):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.STATUS)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        return msg

    async def status_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.status_msg())

    async def accept_proposal(self, context):
        """
        Directs verity-application to accept the proposed presentation.
        verity application will send presentation request based on the proposal.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.accept_proposal_msg_packed(context))

    def accept_proposal_msg(self):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.ACCEPT_PROPOSAL)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        return msg

    async def accept_proposal_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.accept_proposal_msg())

    async def reject(self, context, reason):
        """
        Directs verity-application to reject this presentation proof protocol

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
            reason: the reason for rejection
        """
        await self.send_message(context, await self.reject_msg_packed(context, reason))

    def reject_msg(self, reason):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
            reason: the reason for rejection

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.REJECT)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        if reason:
            msg['reason'] = reason
        return msg

    async def reject_msg_packed(self, context, reason):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
            reason: the reason for rejection

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.reject_msg(reason))



import pytest
from verity_sdk.protocols.v0_6.WriteSchema import WriteSchema
from verity_sdk.utils import EVERNYM_MSG_QUALIFIER
from verity_sdk.utils.Context import Context

from ..test_utils import get_test_config, cleanup

schema_name = 'schema name'
schema_version = '0.1.0'
attrs = ['name', 'degree', 'age']
endorser_did = 'endorserDID'


def test_init():
    write_schema = WriteSchema(schema_name, schema_version, attrs)

    assert write_schema.name == schema_name
    assert write_schema.version == schema_version
    assert len(write_schema.attrs) == 3
    assert write_schema.attrs == list(attrs)

    with pytest.raises(ValueError):
        WriteSchema(None, schema_version, attrs)
    with pytest.raises(ValueError):
        WriteSchema('', schema_version, attrs)
    with pytest.raises(ValueError):
        WriteSchema(schema_name, None, attrs)
    with pytest.raises(ValueError):
        WriteSchema(schema_name, '', attrs)
    with pytest.raises(ValueError):
        WriteSchema(schema_name, schema_version, None)
    with pytest.raises(ValueError):
        WriteSchema(schema_name, schema_version, ['name', None])


@pytest.mark.asyncio
async def test_write():
    context = await Context.create_with_config(await get_test_config())
    write_schema = WriteSchema(schema_name, schema_version, attrs)
    msg = write_schema.write_msg(context)

    assert msg['@type'] == '{}/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        WriteSchema.MSG_FAMILY,
        WriteSchema.MSG_FAMILY_VERSION,
        WriteSchema.WRITE_SCHEMA
    )
    assert msg['@id'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['name'] == schema_name
    assert msg['version'] == schema_version
    assert msg['attrNames'] == list(attrs)


@pytest.mark.asyncio
async def test_write_with_endorser_override():
    context = await Context.create_with_config(await get_test_config())
    write_schema = WriteSchema(schema_name, schema_version, attrs)
    msg = write_schema.write_msg(context, endorser_did)

    assert msg['@type'] == '{}/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        WriteSchema.MSG_FAMILY,
        WriteSchema.MSG_FAMILY_VERSION,
        WriteSchema.WRITE_SCHEMA
    )
    assert msg['@id'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['name'] == schema_name
    assert msg['version'] == schema_version
    assert msg['attrNames'] == list(attrs)
    assert msg['endorserDID'] == endorser_did

    await cleanup(context)


def test_get_message_type():
    write_schema = WriteSchema(schema_name, schema_version, attrs)
    assert write_schema.get_message_type('message_name') == '{}/{}/{}/message_name'.format(
        EVERNYM_MSG_QUALIFIER,
        WriteSchema.MSG_FAMILY,
        WriteSchema.MSG_FAMILY_VERSION
    )



import pytest

from test.test_utils import get_test_config, cleanup
from verity_sdk.protocols.v0_6.WriteCredentialDefinition import WriteCredentialDefinition
from verity_sdk.utils import EVERNYM_MSG_QUALIFIER
from verity_sdk.utils.Context import Context


name = 'name'
schema_id = 'schema id'
tag = 'tag'
revocation_details = {'support_revocation': False}
endorser_did = 'endorserDID'


def test_init():
    write_cred_def = WriteCredentialDefinition(name, schema_id, tag, revocation_details)

    assert write_cred_def.name == name
    assert write_cred_def.schema_id == schema_id
    assert write_cred_def.tag == tag
    assert write_cred_def.revocation_details == revocation_details


@pytest.mark.asyncio
async def test_write():
    context = await Context.create_with_config(await get_test_config())
    write_cred_def = WriteCredentialDefinition(name, schema_id, tag, revocation_details)
    msg = write_cred_def.write_msg(context)

    assert msg['@type'] == '{}/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        WriteCredentialDefinition.MSG_FAMILY,
        WriteCredentialDefinition.MSG_FAMILY_VERSION,
        WriteCredentialDefinition.WRITE_CRED_DEF
    )
    assert msg['@id'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['name'] == name
    assert msg['schemaId'] == schema_id
    assert msg['tag'] == tag
    assert msg['revocationDetails'] == revocation_details


@pytest.mark.asyncio
async def test_write_with_endorser_override():
    context = await Context.create_with_config(await get_test_config())
    write_cred_def = WriteCredentialDefinition(name, schema_id, tag, revocation_details)
    msg = write_cred_def.write_msg(context, endorser_did)

    assert msg['@type'] == '{}/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        WriteCredentialDefinition.MSG_FAMILY,
        WriteCredentialDefinition.MSG_FAMILY_VERSION,
        WriteCredentialDefinition.WRITE_CRED_DEF
    )
    assert msg['@id'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['name'] == name
    assert msg['schemaId'] == schema_id
    assert msg['tag'] == tag
    assert msg['revocationDetails'] == revocation_details
    assert msg['endorserDID'] == endorser_did

    await cleanup(context)



import pytest

from test.test_utils import get_test_config, cleanup
from verity_sdk.protocols.v0_6.UpdateConfigs import UpdateConfigs
from verity_sdk.utils import EVERNYM_MSG_QUALIFIER
from verity_sdk.utils.Context import Context

name = 'Name1'
logo_url = 'http://logo.url'


def test_init():
    update_configs = UpdateConfigs(name, logo_url)

    assert update_configs.name == name
    assert update_configs.logo_url == logo_url


@pytest.mark.asyncio
async def test_update():
    context = await Context.create_with_config(await get_test_config())
    update_configs = UpdateConfigs(name, logo_url)
    msg = update_configs.update_msg(context)

    assert msg['@type'] == '{}/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        UpdateConfigs.MSG_FAMILY,
        UpdateConfigs.MSG_FAMILY_VERSION,
        UpdateConfigs.UPDATE_CONFIGS
    )
    assert msg['@id'] is not None
    assert msg['configs'] == [
        {'name': 'name', 'value': name},
        {'name': 'logoUrl', 'value': logo_url}
    ]

    await cleanup(context)


@pytest.mark.asyncio
async def test_status():
    context = await Context.create_with_config(await get_test_config())
    update_configs = UpdateConfigs()
    msg = update_configs.status_msg(context)

    assert msg['@type'] == '{}/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        UpdateConfigs.MSG_FAMILY,
        UpdateConfigs.MSG_FAMILY_VERSION,
        UpdateConfigs.GET_STATUS
    )
    assert msg['@id'] is not None

    await cleanup(context)



import json


class WalletConfig:
    """
    The generic interface for the wallet configuration for the Indy-SDK wallet
    """

    def config(self) -> str:
        """
        Provides a JSON string that is valid config for the Indy-SDK wallet API
        Returns:
            a JSON wallet config string
        """

    def credential(self) -> str:
        """
        Provides a JSON string that is valid credential for the Indy-SDK wallet API
        Returns:
            a JSON wallet credential string
        """


def build(name, key, path=None):
    """
    Builds a DefaultWalletConfig from the given parameters

    Args:
        name (str): the given name for the wallet to be created
        key (str): the given key for the wallet to be created
        path (str): the given path where the wallet on disk file will be created
    Returns:
        DefaultWalletConfig: constructed with the given parameters
    """
    return DefaultWalletConfig(name, key, path)


class DefaultWalletConfig(WalletConfig):
    """
    The config object for the local default wallet provided by the Indy SDK
    """
    # Cannot use 'id' - shadows build-in name id
    name: str
    key: str
    path: str

    def __init__(self, name: str, key: str, path: str = None):
        """
        Args:
            name (str): the given name or identifier for the wallet
            key (str): the given key (encrypting) for the wallet
            path (str): the given path location where the wallet on disk file is found
        """
        self.name = name
        self.key = key
        self.path = path

    def config(self) -> str:
        """
        Provides a default JSON config string
        Returns:
            str: a JSON wallet config string
        """
        configuration = {
            'id': self.name
        }
        if self.path and self.path.strip:
            configuration['storage_config'] = {
                'path': self.path
            }
        return json.dumps(configuration)

    def credential(self) -> str:
        """
        Provides a default JSON credential string
        Returns:
            str: a JSON wallet credential string
        """
        return json.dumps({'key', self.key})

    def add_to_json(self, target_dict: dict) -> dict:
        """
        Injects the default wallet fields into a JSON Object
        Args:
            target_dict (dict): the JSON object that the fields are injected into
        """
        if self.name:
            target_dict['walletName'] = self.name
        if self.key:
            target_dict['walletKey'] = self.key
        if self.path:
            target_dict['walletPath'] = self.path
        return target_dict



import json
from uuid import uuid4 as uuid
import pytest
from indy import wallet
from verity_sdk.utils import _prepare_forward_message, pack_message_for_verity, \
    unpack_forward_message, get_message_type, get_problem_report_message_type, \
    EVERNYM_MSG_QUALIFIER, get_status_message_type
from verity_sdk.utils.Context import Context
from verity_sdk.utils.Did import create_new_did


async def get_test_config(seed=None):
    test_config = {
        'walletName': str(uuid()),
        'walletKey': str(uuid()),
        'verityUrl': 'http://localhost:8080/agency/msg',
        'verityPublicVerkey': None,
        'verityPairwiseDID': None,
        'verityPairwiseVerkey': None,
        'sdkPairwiseVerkey': None,
        'endpointUrl': 'http://localhost:4000'
    }

    wallet_config = json.dumps({'id': test_config['walletName']})
    wallet_credentials = json.dumps({'key': test_config['walletKey']})
    await wallet.create_wallet(wallet_config, wallet_credentials)
    wallet_handle = await wallet.open_wallet(wallet_config, wallet_credentials)
    their_public = await create_new_did(wallet_handle)
    their = await create_new_did(wallet_handle)
    my = await create_new_did(wallet_handle, seed)
    await wallet.close_wallet(wallet_handle)

    test_config['verityPublicDID'] = their_public.did
    test_config['verityPublicVerkey'] = their_public.verkey
    test_config['verityPairwiseDID'] = their.did
    test_config['verityPairwiseVerkey'] = their.verkey
    test_config['sdkPairwiseDID'] = my.did
    test_config['sdkPairwiseVerkey'] = my.verkey

    return json.dumps(test_config)


async def cleanup(context: Context):
    if not context.wallet_closed:
        await context.close_wallet()
    await wallet.delete_wallet(context.wallet_config, context.wallet_credentials)


@pytest.mark.asyncio
async def test_context():
    test_config = await get_test_config()

    context = await Context.create_with_config(test_config)
    test_config = json.loads(test_config)
    assert context.wallet_name == test_config['walletName']
    assert context.wallet_key == test_config['walletKey']
    assert context.verity_url == test_config['verityUrl']
    assert context.verity_public_verkey == test_config['verityPublicVerkey']
    assert context.domain_did == test_config['verityPairwiseDID']
    assert context.verity_agent_verkey == test_config['verityPairwiseVerkey']
    assert context.sdk_verkey == test_config['sdkPairwiseVerkey']
    assert context.endpoint_url == test_config['endpointUrl']

    await cleanup(context)


@pytest.mark.asyncio
async def test_v01_to_v02():
    wallet_name = uuid()
    wallet_key = uuid()
    v01Str = f"""{{
      "verityPublicVerkey": "ETLgZKeQEKxBW7gXA6FBn7nBwYhXFoogZLCCn5EeRSQV",
      "verityPairwiseDID": "NTvSuSXzygyxWrF3scrhdc",
      "verityUrl": "https://vas-team1.pdev.evernym.com",
      "verityPairwiseVerkey": "ChXRWjQdrrLyksbPQZfaS3JekA4xLgD5Jg7GzXhc9zqE",
      "walletName": "{wallet_name}",
      "walletKey": "{wallet_key}",
      "sdkPairwiseVerkey": "HZ3Ak6pj9ryFASKbA9fpwqjVh42F35UDiCLQ13J58Xoh",
      "verityPublicDID": "Rgj7LVEonrMzcRC1rhkx76",
      "sdkPairwiseDID": "XNRkA8tboikwHD3x1Yh7Uz"
    }}"""

    wallet_config = json.dumps({'id': str(wallet_name)})
    wallet_credentials = json.dumps({'key': str(wallet_key)})
    await wallet.create_wallet(wallet_config, wallet_credentials)

    ctx = await Context.create_with_config(v01Str)
    await ctx.close_wallet()

    assert ctx.domain_did == 'NTvSuSXzygyxWrF3scrhdc'
    assert ctx.verity_agent_verkey == 'ChXRWjQdrrLyksbPQZfaS3JekA4xLgD5Jg7GzXhc9zqE'
    assert ctx.sdk_verkey == 'HZ3Ak6pj9ryFASKbA9fpwqjVh42F35UDiCLQ13J58Xoh'
    assert ctx.sdk_verkey_id == 'XNRkA8tboikwHD3x1Yh7Uz'
    assert ctx.version == '0.2'


def test_prepare_forward_message():
    test_did = '123'
    message = b'{"hello": "world"}'
    msg = _prepare_forward_message(test_did, message)
    msg = json.loads(msg)

    assert msg['@type'] == EVERNYM_MSG_QUALIFIER+'/routing/1.0/FWD'
    assert msg['@fwd'] == test_did
    assert msg['@msg'] == json.loads(message.decode('utf-8'))


@pytest.mark.asyncio
async def test_pack_message_for_verity_and_unpack_forward_message():
    message = {'hello': 'world'}
    context = await Context.create_with_config(await get_test_config())
    packed_message = await pack_message_for_verity(context, message)
    unpacked_message = await unpack_forward_message(context, packed_message)
    assert unpacked_message['hello'] == 'world'

    await cleanup(context)


def test_get_message_type():
    msg_type: str = EVERNYM_MSG_QUALIFIER+'/credential/0.1/status'
    assert get_message_type('credential', '0.1', 'status') == msg_type


def test_get_problem_report_message_type():
    assert get_problem_report_message_type('credential', '0.1') == \
           '{}/credential/0.1/problem-report'.format(EVERNYM_MSG_QUALIFIER)


def test_get_status_message_type():
    assert get_status_message_type('credential', '0.1') == \
           '{}/credential/0.1/status'.format(EVERNYM_MSG_QUALIFIER)



from verity_sdk.protocols.Protocol import Protocol
from verity_sdk.utils import EVERNYM_MSG_QUALIFIER


class IssuerSetup(Protocol):
    """
    The IssuerSetup protocol starts the setup process for issuers of verifiable credentials.
    The main task is creating a public identity that can be used to issue credentials. This identity
    and it's keys are created and held on the verity-application. The public elements (DID and verkey)
    are communicated back via a signal message. The setup process encapsulated by this protocol must
    be completed before other issuing activities (write-schema, write-cred-def and issue-credential)
    can be done.
    """

    MSG_FAMILY = 'issuer-setup'
    """the family name for the message family"""

    MSG_FAMILY_VERSION = '0.6'
    """the version for the message family"""

    # Messages

    CREATE = 'create'
    """Name for 'create' control message"""

    CURRENT_PUBLIC_IDENTIFIER = 'current-public-identifier'
    """Name for 'current-public-identifier' control message"""

    PUBLIC_IDENTIFIER = 'public-identifier'
    """Name for 'public-identifier' signal message"""

    PUBLIC_IDENTIFIER_CREATED = 'public-identifier-created'
    """Name for 'public-identifier-created' signal message"""

    PROBLEM_REPORT = 'problem-report'

    def __init__(self):
        super().__init__(
            self.MSG_FAMILY,
            self.MSG_FAMILY_VERSION,
            msg_qualifier=EVERNYM_MSG_QUALIFIER
        )

    async def create(self, context):
        """
        Directs verity-application to start and create an issuer identity and set it up

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.create_msg_packed(context))

    def create_msg(self, context):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.CREATE)
        self._add_thread(msg)
        return msg

    async def create_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.create_msg(context))

    async def current_public_identifier(self, context):
        """
        Asks the verity-application for the current issuer identity that is setup.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.current_public_identifier_msg_packed(context))

    def current_public_identifier_msg(self, context):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.CURRENT_PUBLIC_IDENTIFIER)
        self._add_thread(msg)
        return msg

    async def current_public_identifier_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.current_public_identifier_msg(context))



class DbcUtil:
    """
    Utilities
    for design by contract.Allows for common pre-condition and post-condition checking.
    """

    @staticmethod
    def require(requirement: bool, msg: str):
        """
        Checks that given requirement is true, throws an ValueError if false
        :param requirement: testable requirement
        :param msg: string that is used to build exception message
        """
        if not requirement:
            raise ValueError('requirement failed: ' + msg)

    @staticmethod
    def require_not_null(arg, arg_name: str):
        """
        Checks and throws an ValueError if the given object is null
        :param arg: any object that can be null
        :param arg_name: name of the argument being checked, used in exception message if null
        """
        DbcUtil.require(arg is not None, 'required that ' + arg_name + ' must NOT be None')

    @staticmethod
    def require_string_not_null_or_empty(arg: str, arg_name: str):
        """
        Checks and throws an ValueError if the given string is null or empty
        :param arg: String argument which needs to be checked
        :param arg_name: name of the argument being checked, used in exception message if null or empty
        """
        DbcUtil.require(arg is not None and len(arg) > 0, 'required that ' + arg_name + ' must NOT be null or empty')

    @staticmethod
    def require_array_not_contain_null(array: list, arg_name: str):
        """
        Checks and throws an ValueError if the given array is null or contains null element
        :param array: Array argument which needs to be checked
        :param arg_name: name of the argument being checked, used in exception message
        """
        DbcUtil.require_not_null(array, arg_name)
        for x in array:
            DbcUtil.require(x, 'required that elements of ' + arg_name + ' must NOT be None')



import pytest

from verity_sdk.protocols.v1_0.PresentProof import PresentProof
from verity_sdk.utils import COMMUNITY_MSG_QUALIFIER

for_relationship = 'some_did'
thread_id = 'some thread id'
name = 'Degree Verification'
proof_attrs = [
    {'name': 'name', 'restrictions': [{'issuer_did': 'did:sov:12345'}]},
    {'name': 'degree', 'restrictions': [{'issuer_did': 'did:sov:12345'}]},
    {'names': ['first_name', 'last_name'], 'restrictions': [{'issuer_did': 'did:sov:12345'}]}
]
proof_predicates = [
    {'name': 'age', 'p_type': 'GT', 'p_value': 18, 'restrictions': [{'issuer_did': 'did:sov:12345'}]}
]
revocation_interval = {'support_revocation': False}
reject_reason = 'because'


def test_init():
    present_proof = PresentProof(for_relationship, thread_id, name, proof_attrs, proof_predicates)

    assert present_proof.for_relationship == for_relationship
    assert present_proof.thread_id == thread_id
    assert present_proof.name == name
    assert present_proof.proof_attrs == proof_attrs
    assert present_proof.proof_predicates == proof_predicates


@pytest.mark.asyncio
async def test_request():
    present_proof = PresentProof(for_relationship, None, name, proof_attrs, proof_predicates)
    msg = present_proof.request_msg()

    assert msg['@type'] == '{}/{}/{}/{}'.format(
        COMMUNITY_MSG_QUALIFIER,
        PresentProof.MSG_FAMILY,
        PresentProof.MSG_FAMILY_VERSION,
        PresentProof.PROOF_REQUEST
    )
    assert msg['@id'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['name'] == name
    assert msg['proof_attrs'] == proof_attrs
    assert msg['proof_predicates'] == proof_predicates


@pytest.mark.asyncio
async def test_status():
    present_proof = PresentProof(for_relationship, thread_id)
    msg = present_proof.status_msg()

    assert msg['@type'] == '{}/{}/{}/{}'.format(
        COMMUNITY_MSG_QUALIFIER,
        PresentProof.MSG_FAMILY,
        PresentProof.MSG_FAMILY_VERSION,
        PresentProof.STATUS
    )
    assert msg['@id'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] == thread_id


@pytest.mark.asyncio
async def test_accept_proposal():
    present_proof = PresentProof(for_relationship, thread_id)
    msg = present_proof.accept_proposal_msg()

    assert msg['@type'] == '{}/{}/{}/{}'.format(
        COMMUNITY_MSG_QUALIFIER,
        PresentProof.MSG_FAMILY,
        PresentProof.MSG_FAMILY_VERSION,
        PresentProof.ACCEPT_PROPOSAL
    )
    assert msg['@id'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] == thread_id


@pytest.mark.asyncio
async def test_reject():
    present_proof = PresentProof(for_relationship, thread_id)
    msg = present_proof.reject_msg(reject_reason)

    assert msg['@type'] == '{}/{}/{}/{}'.format(
        COMMUNITY_MSG_QUALIFIER,
        PresentProof.MSG_FAMILY,
        PresentProof.MSG_FAMILY_VERSION,
        PresentProof.REJECT
    )
    assert msg['@id'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] == thread_id
    assert msg['reason'] == reject_reason



from enum import Enum

from verity_sdk.protocols.Protocol import Protocol
from verity_sdk.utils import EVERNYM_MSG_QUALIFIER, Context


class GoalsList(Enum):
    """An enumeration of possible goals (reasons) for a relationship."""

    class GoalCode:
        """
        a tuple with a short code string and human readable name string
        """

        def __init__(self, code, name) -> None:
            """
            Args:
                code (str): short code string
                name (str): human readable string
            """
            self.code = code
            self.name = name

    ISSUE_VC = GoalCode('issue-vc', 'To issue a credential')
    """To issue a credential"""

    REQUEST_PROOF = GoalCode('request-proof', 'To request a proof')
    """To request a proof"""

    CREATE_ACCOUNT = GoalCode('create-account', 'To create an account with a service')
    """To create an account with a service"""

    P2P_MESSAGING = GoalCode('p2p-messaging', 'To establish a peer-to-peer messaging relationship')
    """To establish a peer-to-peer messaging relationship"""


class Relationship(Protocol):
    """
    The Relationship protocol creates and manages relationships on the verity-application agent. These relationships
    are secure communication channels between self-sovereign parties. A relationship much be created before using the
    connections protocols. In the future this protocol will allow management of each given relationship
    (eg key rotation)
    """
    MSG_FAMILY = 'relationship'
    """the family name for the message family"""

    MSG_FAMILY_VERSION = '1.0'
    """the version for the message family"""

    CREATED = 'created'
    """Name for 'created' signal message"""
    INVITATION = 'invitation'
    """Name for 'invitation' signal message"""

    CREATE = 'create'
    """Name for 'create' control message"""
    CONNECTION_INVITATION = 'connection-invitation'
    """Name for 'connection-invitation' control message"""
    OUT_OF_BAND_INVITATION = 'out-of-band-invitation'
    """Name for 'out-of-band-invitation' control message"""
    SMS_CONNECTION_INVITATION = 'sms-connection-invitation'
    """Name for 'sms-connection-invitation' control message"""
    SMS_OUT_OF_BAND_INVITATION = 'sms-out-of-band-invitation'
    """Name for 'sms-out-of-band-invitation' control message"""

    def __init__(self,
                 for_relationship: str = None,
                 thread_id: str = None,
                 label: str = None,
                 logo_url: str = None,
                 ):
        """
        Args:
            for_relationship (str): the relationship identifier (DID) for the pairwise relationship that will be used
            thread_id (str): the thread id of the already started protocol
            label (str): he label presented in the invitation to connect to this relationship
            logo_url (str): logo url presented in invitation
        """
        super().__init__(
            self.MSG_FAMILY,
            self.MSG_FAMILY_VERSION,
            msg_qualifier=EVERNYM_MSG_QUALIFIER,
            thread_id=thread_id
        )

        self.for_relationship = for_relationship
        self.label = label
        self.logo_url = logo_url

    async def create(self, context):
        """
        Directs verity-application to create a new relationship

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.create_msg_packed(context))

    def create_msg(self, context: Context):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.CREATE)
        self._add_thread(msg)
        if self.label:
            msg['label'] = self.label
        if self.logo_url:
            msg['logoUrl'] = self.logo_url

        return msg

    async def create_msg_packed(self, context: Context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.create_msg(context))

    async def connection_invitation(self, context: Context, short_invite: bool = None):
        """
        Ask for aries invitation from the verity-application agent for the relationship created by this protocol

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
            short_invite (bool): Request the short invite
        """
        await self.send_message(context, await self.connection_invitation_msg_packed(context, short_invite))

    def connection_invitation_msg(self, context: Context, short_invite: bool = None):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
            short_invite (bool): Request the short invite
        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.CONNECTION_INVITATION)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        if short_invite is not None:
            msg['shortInvite'] = short_invite
        return msg

    async def connection_invitation_msg_packed(self, context: Context, short_invite: bool = None):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
            short_invite (bool): Request the short invite
        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.connection_invitation_msg(context, short_invite))

    async def sms_connection_invitation(self, context: Context, phone_number: str):
        """
        Ask for SMS aries invitation from the verity-application agent for the relationship created by this protocol

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
            phone_number (str): Mobile phone number in international format, eg. +18011234567
        """
        await self.send_message(context, await self.sms_connection_invitation_msg_packed(context, phone_number))

    def sms_connection_invitation_msg(self, context: Context, phone_number: str):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
            phone_number (str): Mobile phone number in international format, eg. +18011234567
        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.SMS_CONNECTION_INVITATION)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        msg['phoneNumber'] = phone_number
        return msg

    async def sms_connection_invitation_msg_packed(self, context: Context, phone_number: str):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
            phone_number (str): Mobile phone number in international format, eg. +18011234567
        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.sms_connection_invitation_msg(context, phone_number))

    async def out_of_band_invitation(self,
                                     context: Context,
                                     short_invite: bool = None,
                                     goal: GoalsList = GoalsList.P2P_MESSAGING):
        """
        Ask for aries out of band invitation from the verity-application agent for the relationship
        created by this protocol

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
            short_invite (bool): Request the short invite
            goal (GoalsList): the initial intended goal of the relationship (this goal is expressed in the invite)
        """
        await self.send_message(context, await self.out_of_band_invitation_msg_packed(context, short_invite, goal))

    def out_of_band_invitation_msg(self,
                                   context: Context,
                                   short_invite: bool = None,
                                   goal: GoalsList = None):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
            short_invite (bool): Request the short invite
            goal (GoalsList): the initial intended goal of the relationship (this goal is expressed in the invite)
        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.OUT_OF_BAND_INVITATION)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        if short_invite is not None:
            msg['shortInvite'] = short_invite

        if goal:
            msg['goalCode'] = goal.value.code
            msg['goal'] = goal.value.name

        return msg

    async def out_of_band_invitation_msg_packed(self,
                                                context: Context,
                                                short_invite: bool = None,
                                                goal: GoalsList = None):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
            short_invite (bool): Request the short invite
            goal (GoalsList): the initial intended goal of the relationship (this goal is expressed in the invite)
        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.out_of_band_invitation_msg(context, short_invite, goal))

    async def sms_out_of_band_invitation(self,
                                         context: Context,
                                         phone_number: str,
                                         goal: GoalsList = None):
        """
        Ask for SMS aries out of band invitation from the verity-application agent for the relationship
        created by this protocol

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
            phone_number (str): Mobile phone number in international format, eg. +18011234567
            goal (GoalsList): the initial intended goal of the relationship (this goal is expressed in the invite)
        """
        await self.send_message(context, await self.sms_out_of_band_invitation_msg_packed(context, phone_number, goal))

    def sms_out_of_band_invitation_msg(self,
                                       context: Context,
                                       phone_number: str,
                                       goal: GoalsList = None):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
            phone_number (str): Mobile phone number in international format, eg. +18011234567
            goal (GoalsList): the initial intended goal of the relationship (this goal is expressed in the invite)
        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.SMS_OUT_OF_BAND_INVITATION)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)

        msg['phoneNumber'] = phone_number
        if goal:
            msg['goalCode'] = goal.value.code
            msg['goal'] = goal.value.name

        return msg

    async def sms_out_of_band_invitation_msg_packed(self,
                                                    context: Context,
                                                    phone_number: str,
                                                    goal: GoalsList = None):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
            phone_number (str): Mobile phone number in international format, eg. +18011234567
            goal (GoalsList): the initial intended goal of the relationship (this goal is expressed in the invite)
        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.sms_out_of_band_invitation_msg(context, phone_number, goal))



import pytest

from test.test_utils import get_test_config
from verity_sdk.protocols.v0_7.Provision import Provision
from verity_sdk.utils.Context import Context
from verity_sdk.utils import EVERNYM_MSG_QUALIFIER


@pytest.mark.asyncio
async def test_message():
    context = await Context.create_with_config(await get_test_config())
    p = Provision()

    msg = p.provision_msg(context)

    assert msg['requesterVk'] is not None
    assert msg['requesterVk'] == context.sdk_verkey
    assert msg['@type'] == EVERNYM_MSG_QUALIFIER+'/agent-provisioning/0.7/create-edge-agent'

    test_token = """{"sponseeId": "myId", "sponsorId": "evernym-test-sponsorabc123", "nonce": "123", "timestamp":
    "2020-06-05T21:33:36.085Z", "sig":
    "ZkejifRr3txh7NrKokC5l2D2YcABUlGlAoFHZD9RapHHBfVtNnHgYux1RCAiEh4Q31VJE3C92T1ZnqDm1WlEAA==", "sponsorVerKey":
    "GJ1SzoWzavQYfNL9XkaJdrQejfztN4XqdsiV4ct3LXKL"} """
    p = Provision(test_token)

    msg = p.provision_msg(context)

    assert msg['provisionToken'] is not None


@pytest.mark.asyncio
async def test_process_message():
    context = await Context.create_with_config(await get_test_config())
    p = Provision()

    async def mock_send(_, __):
        return {
            '@type': EVERNYM_MSG_QUALIFIER+'/agent-provisioning/0.7/AGENT_CREATED',
            'selfDID': '4ut8tgCBdUMCYZrJh5JS4o',
            'agentVerKey': '38fBEUX2sUQF7ZsY6epz1cbCvfBfDbQghUZUzFP6JChE'
        }

    # pylint: disable=protected-access
    p._send_to_verity = mock_send

    context2 = await p.provision(context)
    assert context2.domain_did == '4ut8tgCBdUMCYZrJh5JS4o'
    assert context2.verity_agent_verkey == '38fBEUX2sUQF7ZsY6epz1cbCvfBfDbQghUZUzFP6JChE'


@pytest.mark.asyncio
async def test_validate_token():
    test_token = """{"sponseeId": "myId", "sponsorId": "evernym-test-sponsorabc123", "nonce": "123", "timestamp":
    "2020-06-05T21:33:36.085Z", "sig":
    "ZkejifRr3txh7NrKokC5l2D2YcABUlGlAoFHZD9RapHHBfVtNnHgYux1RCAiEh4Q31VJE3C92T1ZnqDm1WlEAA==", "sponsorVerKey":
    "GJ1SzoWzavQYfNL9XkaJdrQejfztN4XqdsiV4ct3LXKL"} """
    await Provision.validate_token(test_token)

    test_token = """{"sponseeId": "myId", "sponsorId": "evernym-test-sponsorabc123", "nonce": "123", "timestamp":
    "2020-06-05T21:33:36.085Z", "sig":
    "AkejifRr3txh7NrKokC5l2D2YcABUlGlAoFHZD9RapHHBfVtNnHgYux1RCAiEh4Q31VJE3C92T1ZnqDm1WlEAA==", "sponsorVerKey":
    "GJ1SzoWzavQYfNL9XkaJdrQejfztN4XqdsiV4ct3LXKL"} """
    with pytest.raises(Exception):
        await Provision.validate_token(test_token)



# The annotation @AddHandler was not maintained when we refactored the handlers so I'm disabling it for now.
#
# import pytest
#
# from verity_sdk.handlers import is_problem_report, Handlers, AddHandler
#
#
# def test_is_problem_report():
#     assert is_problem_report('vs.service/enroll/0.1/problem-report')
#     assert not is_problem_report('vs.service/enroll/0.1/enroll')
#
#
# # pylint: disable=unused-variable
# @pytest.mark.asyncio
# async def test_decorator():
#     handlers = Handlers()
#
#     @AddHandler(handlers, 'some type', 0)
#     async def some_handler(msg: dict) -> None:
#         print('do something with {}'.format(msg))
#
#     @AddHandler(handlers, 'another type', 1)
#     async def another_handler(msg: dict) -> None:
#         print('do another thing with {}'.format(msg))
#
#     assert len(handlers.handlers) == 2
#     assert handlers.handlers[0].message_type == 'some type'
#     assert handlers.handlers[0].message_status == 0
#     await handlers.handlers[0]({})
#     assert handlers.handlers[1].message_type == 'another type'
#     assert handlers.handlers[1].message_status == 1
#     await handlers.handlers[1]({})



import json
from urllib.parse import urljoin

import requests
from base58 import b58encode

from indy import wallet, crypto

from verity_sdk.utils import Did
from verity_sdk.utils.Wallet import create_and_open_wallet

V_0_1 = '0.1'
V_0_2 = '0.2'


class Context:
    """
    This Context object holds the data for accessing an agent on a verity-application. A complete and correct data in
    the context allows for access and authentication to that agent.
    """
    verity_url: str
    verity_public_did: str
    verity_public_verkey: str
    domain_did: str
    verity_agent_verkey: str
    sdk_verkey_id: str
    sdk_verkey: str
    endpoint_url: str
    wallet_name: str
    wallet_path: str
    wallet_key: str
    wallet_config: str
    wallet_credentials: str
    wallet_handle: int
    wallet_closed: bool
    version: str = V_0_2

    @classmethod
    # pylint: disable=too-many-arguments
    # pylint: disable=protected-access
    async def create(cls,
                     wallet_name: str,
                     wallet_key: str,
                     verity_url: str,
                     domain_did: str = None,
                     verity_agent_verkey: str = None,
                     endpoint_url: str = None,
                     seed: str = None,
                     wallet_path: str = None):
        """
        Creates a new Context with the given parameters.

        Args:
            wallet_name (str): the id for the local wallet
            wallet_key (str): the key (credential) for the local wallet
            verity_url (str): the url for the targeted instance of the verity-application
            domain_did (str): the domain DID for the agent already provisioned
            verity_agent_verkey (str): the verkey for the agent already provisioned
            endpoint_url (str): the given endpoint url
            seed (str): the seed used to generate the local key-pair
            wallet_path (str): the given path where the wallet on disk file will be created
        Returns:
            Context: constructed Context
        """
        context = cls()
        context._set_wallet_config(wallet_name, wallet_path)
        context._set_wallet_credentials(wallet_key)
        context.wallet_name = wallet_name
        context.wallet_key = wallet_key
        context.wallet_path = wallet_path
        context.verity_url = verity_url

        await context._update_verity_info()
        context.wallet_handle = await create_and_open_wallet(
            context.wallet_config,
            context.wallet_credentials
        )

        context.endpoint_url = endpoint_url

        context.domain_did = domain_did

        context.verity_agent_verkey = verity_agent_verkey

        sdk_key = await Did.create_new_did(context.wallet_handle, seed=seed)

        context.sdk_verkey_id = sdk_key.did
        context.sdk_verkey = sdk_key.verkey

        return context

    def _set_wallet_config(self, wallet_name, wallet_path=None):
        wallet_config = {'id': wallet_name}

        if wallet_path:
            wallet_config['storage_config'] = {'path': wallet_path}

        self.wallet_config = json.dumps(wallet_config)

    def _set_wallet_credentials(self, wallet_key):
        self.wallet_credentials = json.dumps({'key': wallet_key})

    async def _update_verity_info(self):
        full_url = urljoin(self.verity_url, 'agency')
        result = requests.get(full_url)
        status_code = result.status_code
        if status_code > 399:
            raise IOError(f'Request failed! - {status_code} - {result.text}')

        try:
            msg = result.json()
            self.verity_public_did = msg.get('DID')
            self.verity_public_verkey = msg.get('verKey')
        except ValueError as e:
            raise IOError(f'Invalid and unexpected data from Verity -- response -- {e}')

    @classmethod
    async def create_with_config(cls, config: str):
        """
        Loads the fields contained in the given JSON Object into this ContextBuilder. The JSON Object should be
        from a Context object that was converted to JSON
        Args:
            config (str): a str that contains properly formatted JSON
        Returns:
            Context: this ContextBuilder with loaded fields
        """
        context = cls()
        config = json.loads(config)

        version = config.get('version', V_0_1)

        if V_0_1 == version:
            context = cls._parse_v01(context, config)
        elif V_0_2 == version:
            context = cls._parse_v02(context, config)
        else:
            raise Exception(f'Invalid context version -- "{version}" is not supported')

        context.wallet_handle = await wallet.open_wallet(
            context.wallet_config,
            context.wallet_credentials
        )

        context.wallet_closed = False

        return context

    @classmethod
    # pylint: disable=protected-access
    def _parse_v01(cls, context, config):
        context.wallet_name = config['walletName']
        context.wallet_key = config['walletKey']
        context.wallet_path = config.get('walletPath')
        context.verity_url = config['verityUrl']
        context.verity_public_did = config.get('verityPublicDID')
        context.verity_public_verkey = config.get('verityPublicVerkey')
        context.domain_did = config.get('verityPairwiseDID')
        context.verity_agent_verkey = config.get('verityPairwiseVerkey')
        context.sdk_verkey_id = config.get('sdkPairwiseDID')
        context.sdk_verkey = config.get('sdkPairwiseVerkey')
        context.endpoint_url = config.get('endpointUrl')

        context._set_wallet_config(context.wallet_name, context.wallet_path)
        context._set_wallet_credentials(context.wallet_key)

        return context

    @classmethod
    # pylint: disable=protected-access
    def _parse_v02(cls, context, config):
        context.wallet_name = config['walletName']
        context.wallet_key = config['walletKey']
        context.wallet_path = config.get('walletPath')
        context.verity_url = config['verityUrl']

        context.verity_public_did = config.get('verityPublicDID')
        context.verity_public_verkey = config.get('verityPublicVerKey')

        context.domain_did = config.get('domainDID')
        context.verity_agent_verkey = config.get('verityAgentVerKey')

        context.sdk_verkey_id = config.get('sdkVerKeyId')
        context.sdk_verkey = config.get('sdkVerKey')

        context.endpoint_url = config.get('endpointUrl')

        context._set_wallet_config(context.wallet_name, context.wallet_path)
        context._set_wallet_credentials(context.wallet_key)

        return context

    async def close_wallet(self):
        """
        Closes the the open wallet handle stored inside the Context object.
        """
        await wallet.close_wallet(self.wallet_handle)
        self.wallet_closed = True

    def to_json(self, indent=None) -> str:
        """
        Converts this object into a JSON object
        Args:
            indent (int): number to indent for pretty printing
        Returns:
            str: a string (JSON encoded) based on this context
        """
        return json.dumps(
            {
                'walletName': self.wallet_name,
                'walletKey': self.wallet_key,
                'walletPath': self.wallet_path if hasattr(self, 'wallet_path') else None,
                'verityUrl': self.verity_url,
                'verityPublicDID': self.verity_public_did,
                'verityPublicVerKey': self.verity_public_verkey,
                'domainDID': self.domain_did,
                'verityAgentVerKey': self.verity_agent_verkey,
                'sdkVerKeyId': self.sdk_verkey_id,
                'sdkVerKey': self.sdk_verkey,
                'endpointUrl': self.endpoint_url,
                'version': V_0_2,
            },
            indent=indent
        )

    async def rest_api_token(self) -> str:
        """
        Converts the local keys held in the context to REST api token. This token can be used with the REST API for the
        Return:
            str: a REST API token
        """
        verkey = self.sdk_verkey
        b = await crypto.crypto_sign(self.wallet_handle, verkey, verkey.encode('utf-8'))
        t = b58encode(b).decode('utf-8')
        return verkey + ':' + t



# import json
#
# import pytest
#
# from test.test_utils import get_test_config, cleanup
# from verity_sdk.protocols.v1_0.IssueCredential import IssueCredential
# from verity_sdk.utils import EVERNYM_MSG_QUALIFIER
# from verity_sdk.utils.Context import Context
#
# for_relationship = 'some_did'
# name = 'degree'
# cred_def_id = '12345'
# credential_values = {
#     'name': 'Joe',
#     'degree': 'Bachelors',
#     'gpa': '3.67'
# }
# price = '5'
#
#
# def test_init():
#     issue_credential = IssueCredential(for_relationship, None, name, cred_def_id, credential_values, price)
#
#     assert issue_credential.for_relationship == for_relationship
#     assert issue_credential.name == name
#     assert issue_credential.cred_def_id == cred_def_id
#     assert json.dumps(issue_credential.credential_values) == json.dumps(credential_values)
#
#
# @pytest.mark.asyncio
# async def test_offer_credential():
#     context = await Context.create_with_config(await get_test_config())
#     issue_credential = IssueCredential(for_relationship, None, name, cred_def_id, credential_values, price)
#     msg = issue_credential.offer_credential_msg(context)
#
#     assert msg['@type'] == '{}/{}/{}/{}'.format(
#         EVERNYM_MSG_QUALIFIER,
#         IssueCredential.MSG_FAMILY,
#         IssueCredential.MSG_FAMILY_VERSION,
#         IssueCredential.OFFER_CREDENTIAL
#     )
#     assert msg['@id'] is not None
#     assert msg['~for_relationship'] == for_relationship
#     assert msg['~thread'] is not None
#     assert msg['~thread']['thid'] is not None
#     assert msg['name'] == name
#     assert msg['credDefId'] == cred_def_id
#     assert msg['credentialValues']['name'] == credential_values['name']
#     assert msg['credentialValues']['degree'] == credential_values['degree']
#     assert msg['credentialValues']['gpa'] == credential_values['gpa']
#     assert msg['price'] == price
#
#     await cleanup(context)
#
#
# @pytest.mark.asyncio
# async def test_issue_credential():
#     context = await Context.create_with_config(await get_test_config())
#     issue_credential = IssueCredential(for_relationship, None, name, cred_def_id, credential_values, price)
#     msg = issue_credential.issue_credential_msg(context)
#
#     assert msg['@type'] == '{}/{}/{}/{}'.format(
#         EVERNYM_MSG_QUALIFIER,
#         IssueCredential.MSG_FAMILY,
#         IssueCredential.MSG_FAMILY_VERSION,
#         IssueCredential.ISSUE_CREDENTIAL
#     )
#     assert msg['@id'] is not None
#     assert msg['~for_relationship'] == for_relationship
#     assert msg['~thread'] is not None
#     assert msg['~thread']['thid'] is not None
#
#     await cleanup(context)
#
#
# @pytest.mark.asyncio
# async def test_status():
#     context = await Context.create_with_config(await get_test_config())
#     issue_credential = IssueCredential(for_relationship, None, name, cred_def_id, credential_values, price)
#     msg = issue_credential.status_msg(context)
#
#     assert msg['@type'] == '{}/{}/{}/{}'.format(
#         EVERNYM_MSG_QUALIFIER,
#         IssueCredential.MSG_FAMILY,
#         IssueCredential.MSG_FAMILY_VERSION,
#         IssueCredential.GET_STATUS
#     )
#     assert msg['@id'] is not None
#     assert msg['~for_relationship'] == for_relationship
#     assert msg['~thread'] is not None
#     assert msg['~thread']['thid'] is not None
#
#     await cleanup(context)



from verity_sdk.transports import send_packed_message
from verity_sdk.utils import pack_message_for_verity, uuid
from verity_sdk.utils.MessageFamily import MessageFamily


class Protocol(MessageFamily):
    """The base class for all protocols"""

    # Messages
    STATUS = 'status-report'
    """Name for 'status-report' signal message"""
    PROBLEM_REPORT = 'problem-report'
    """Name for 'problem-report' signal message"""

    def __init__(self, msg_family: str, msg_family_version: str, msg_qualifier: str = None, thread_id: str = None):
        """
        Args:
            msg_family (str): the family name for the message family
            msg_family_version (str): the version for the message family
            msg_qualifier (str): the qualifier for the message family
            thread_id (str): given ID used for the thread.
        """
        super().__init__(msg_family, msg_family_version, msg_qualifier)
        if thread_id is not None:
            self.thread_id = thread_id
        else:
            self.thread_id = uuid()

    def _add_thread(self, msg):
        msg['~thread'] = {
            'thid': self.thread_id
        }

    @staticmethod
    def _add_relationship(msg, for_relationship):
        msg['~for_relationship'] = for_relationship

    @staticmethod
    async def get_message_bytes(context, message) -> bytes:
        """
        Packs the connection message for the verity
        Args:
            context (Context): an instance of Context that has been initialized with your wallet and key details
            message (dict): the message to be packed for the verity-application
        Returns:
            bytes: Encrypted connection message ready to be sent to the verity
        """
        return await pack_message_for_verity(context, message)

    @staticmethod
    async def send_message(context, packed_message):
        """
        Sends a given packed message to Verity

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
            packed_message (bytes):  the encrypted message bytes to send to Verity
        """
        send_packed_message(context, packed_message)



import pytest

from test.test_utils import get_test_config, cleanup
from verity_sdk.protocols.v0_6.IssuerSetup import IssuerSetup
from verity_sdk.utils import EVERNYM_MSG_QUALIFIER
from verity_sdk.utils.Context import Context


def test_init():
    IssuerSetup()


@pytest.mark.asyncio
async def test_create():
    context = await Context.create_with_config(await get_test_config())
    issuer_setup = IssuerSetup()
    msg = issuer_setup.create_msg(context)

    assert msg['@type'] == '{}/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        IssuerSetup.MSG_FAMILY,
        IssuerSetup.MSG_FAMILY_VERSION,
        IssuerSetup.CREATE
    )
    assert msg['@id'] is not None

    await cleanup(context)



from verity_sdk.protocols.Protocol import Protocol
from verity_sdk.utils import EVERNYM_MSG_QUALIFIER


class UpdateEndpoint(Protocol):
    """ The UpdateEndpoint protocol allow changes to the endpoint register with the verity-application agent.
    This endpoint is where agent sends signal messages. These messages are asynchronous and are sent via a http
    web hook. The endpoint is defined in the context. It must be set in the context before using this protocol.
    """

    MSG_FAMILY = 'configs'
    """the family name for the message family"""

    MSG_FAMILY_VERSION = '0.6'
    """the version for the message family"""

    # Messages
    UPDATE_ENDPOINT = 'UPDATE_COM_METHOD'
    """Name for 'update' control message"""

    def __init__(self, authentication=None):
        """
        Args:
            authentication (Dict): Authentication for the endpoint url.
                Please consult documentation about this parameter.
                eg:
                    {
                        'type': 'OAuth2',
                        'version': 'v1',
                        'data': {
                            'url': 'https://auth.url/token',
                            'grant_type': 'client_credentials',
                            'client_id': 'ajeyKDizsDkwYeEmRmHU78gf7W3VIEoA',
                            'client_secret': 'aaGxxcGi6kb6AxIe'
                        }
                    }
        """
        super().__init__(
            self.MSG_FAMILY,
            self.MSG_FAMILY_VERSION,
            msg_qualifier=EVERNYM_MSG_QUALIFIER,
        )
        self.authentication = authentication

    async def update(self, context):
        """
        Directs verity-application to update the used endpoint for out-going signal message to the endpoint contained
        in the context object.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        bytes_to_send = await self.update_endpoint_msg_packed(context)
        await self.send_message(context, bytes_to_send)

    def update_endpoint_msg(self, context):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """

        COM_METHOD_TYPE = 2
        if context.endpoint_url is None or context.endpoint_url == '':
            raise Exception('Unable to update endpoint because context.endpoint_url is not defined')
        msg = self._get_base_message(self.UPDATE_ENDPOINT)
        msg['comMethod'] = {
            'id': 'webhook',
            'type': COM_METHOD_TYPE,
            'value': context.endpoint_url,
            'packaging': {
                'pkgType': '1.0',
                'recipientKeys': [context.sdk_verkey]
            }
        }
        if self.authentication is not None:
            msg['comMethod']['authentication'] = self.authentication
        return msg

    async def update_endpoint_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """

        msg = self.update_endpoint_msg(context)
        return await self.get_message_bytes(context, msg)



import pytest

from test.test_utils import get_test_config, cleanup
from verity_sdk.protocols.v1_0.CommittedAnswer import CommittedAnswer
from verity_sdk.utils import COMMUNITY_MSG_QUALIFIER
from verity_sdk.utils.Context import Context

for_relationship = 'some_did'
question_text = 'Are you trying to login to acme.com?'
question_detail = 'IP Address: 56.24.11.126'
valid_responses = ['Yes', 'No, that\'s not me!']
signature_required = True


def test_init():
    committed_answer = CommittedAnswer(for_relationship, None, question_text, question_detail, valid_responses,
                                       signature_required)

    assert committed_answer.for_relationship == for_relationship
    assert committed_answer.question == question_text
    assert committed_answer.descr == question_detail
    assert committed_answer.valid_responses == valid_responses
    assert committed_answer.signature_required == signature_required


@pytest.mark.asyncio
async def test_ask():
    context = await Context.create_with_config(await get_test_config())
    committed_answer = CommittedAnswer(for_relationship, None, question_text, question_detail, valid_responses,
                                       signature_required)
    msg = committed_answer.ask_msg(context)

    assert msg['@type'] == '{}/{}/{}/{}'.format(
        COMMUNITY_MSG_QUALIFIER,
        CommittedAnswer.MSG_FAMILY,
        CommittedAnswer.MSG_FAMILY_VERSION,
        CommittedAnswer.ASK_QUESTION
    )
    assert msg['@id'] is not None
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['text'] == question_text
    assert msg['detail'] == question_detail
    assert msg['valid_responses'] == valid_responses
    assert msg['signature_required'] == signature_required

    await cleanup(context)


@pytest.mark.asyncio
async def test_status():
    context = await Context.create_with_config(await get_test_config())
    committed_answer = CommittedAnswer(for_relationship, None, question_text, question_detail, valid_responses,
                                       signature_required)
    msg = committed_answer.status_msg(context)

    assert msg['@type'] == '{}/{}/{}/{}'.format(
        COMMUNITY_MSG_QUALIFIER,
        CommittedAnswer.MSG_FAMILY,
        CommittedAnswer.MSG_FAMILY_VERSION,
        CommittedAnswer.GET_STATUS
    )
    assert msg['@id'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None

    await cleanup(context)



import json

from typing import Dict
from uuid import uuid4

from indy import crypto
from verity_sdk.utils.Context import Context

USE_NEW_QUALIFIER_FORMAT = False

EVERNYM_MSG_QUALIFIER = 'https://didcomm.evernym.com' if USE_NEW_QUALIFIER_FORMAT else 'did:sov:123456789abcdefghi1234;spec'
"""QUALIFIER for evernym specific protocols"""
COMMUNITY_MSG_QUALIFIER = 'https://didcomm.org' if USE_NEW_QUALIFIER_FORMAT  else 'did:sov:BzCbsNYhMrjHiqZDTUASHg;spec'
"""QUALIFIER for community specified protocol"""


def _prepare_forward_message(did: str, message: bytes) -> str:
    return json.dumps({
        '@type': EVERNYM_MSG_QUALIFIER+'/routing/1.0/FWD',
        '@fwd': did,
        '@msg': json.loads(message.decode('utf-8'))
    })


async def pack_message_for_verity(context: Context, message: dict) -> bytes:
    """
    Packages message (instructor and encryption) for the verity-application. Uses local private keys and remote
    public keys for encryption. The encryption and instructor is defined by the Aries community.

    Args:
        context (Context): an instance of the Context object initialized to a verity-application agent
        message (dict): the JSON message to be communicated to the verity-application
    Return:
        bytes: the byte array of the packaged and encrypted message
    """
    return await _pack_message_for_verity_direct(
        wallet_handle=context.wallet_handle,
        message=message,
        pairwise_remote_did=context.domain_did,
        pairwise_remote_verkey=context.verity_agent_verkey,
        pairwise_local_verkey=context.sdk_verkey,
        public_verkey=context.verity_public_verkey
    )


async def _pack_message_for_verity_direct(wallet_handle: int,
                                          message: dict,
                                          pairwise_remote_did: str,
                                          pairwise_remote_verkey: str,
                                          pairwise_local_verkey: str,
                                          public_verkey: str) -> bytes:
    # Special packaging for provisioning (for a message before the agent has been provisioned in the verity-application)
    # This should ONLY be needed for the provisioning message
    agent_message = await crypto.pack_message(
        wallet_handle,
        json.dumps(message),
        [pairwise_remote_verkey],
        pairwise_local_verkey,
    )
    forward_message = _prepare_forward_message(
        pairwise_remote_did,
        agent_message
    )
    return await crypto.pack_message(
        wallet_handle,
        forward_message,
        [public_verkey],
        None
    )


async def unpack_forward_message(context: Context, message: bytes) -> Dict:
    """
    Deprecated!
    """
    unpacked_once_message = await unpack_message(context, message)
    return await unpack_message(
        context,
        json.dumps(unpacked_once_message['@msg']).encode('utf-8')
    )


async def unpack_message(context: Context, message: bytes) -> Dict:
    """
    Extracts the message in the byte array that has been packaged and encrypted for a key that is locally held.
    Args:
        context (Context): an instance of the Context object initialized to a verity-application agent
        message (bytes): the raw message received from the verity-application agent
    Returns:
        dict: an unencrypted messages as a JSON object
    """
    jwe: bytes = await crypto.unpack_message(
        context.wallet_handle,
        message
    )
    message = json.loads(jwe.decode('utf-8'))['message']
    return json.loads(message)


def uuid() -> str:
    """
    Returns:
         str: a new random uuid
    """
    return str(uuid4())


# TODO: Remove this function in favor of using MessageFamily
def get_message_type(msg_family: str,
                     msg_family_version: str,
                     msg_name: str,
                     msg_qualifier=EVERNYM_MSG_QUALIFIER) -> str:
    """
    Deprecated!

    See: MesssageFamily.message_type
    """
    return '{}/{}/{}/{}'.format(msg_qualifier, msg_family, msg_family_version, msg_name)


def get_problem_report_message_type(msg_family: str, msg_family_version: str) -> str:
    """
    Deprecated!
    """
    return get_message_type(msg_family, msg_family_version, 'problem-report')


def get_status_message_type(msg_family: str, msg_family_version: str) -> str:
    """
    Deprecated!
    """
    return get_message_type(msg_family, msg_family_version, 'status')


class MsgType:
    def __init__(self, msgType: str):
        parts1 = msgType.split(';spec/')
        self.msg_qualifier = parts1[0]
        parts2 = parts1[1].split('/')
        self.msg_family = parts2[0]
        self.msg_family_version = parts2[1]
        self.msg_name = parts2[2]



import pytest

from test.test_utils import get_test_config, cleanup
from verity_sdk.protocols.v1_0.BasicMessage import BasicMessage
from verity_sdk.utils import COMMUNITY_MSG_QUALIFIER
from verity_sdk.utils.Context import Context

for_relationship = 'some_did'
content = 'Are you trying to login to acme.com?'
sent_time = '12:45:00-000'
localization = 'en'

def test_init():
    basic_message = BasicMessage(for_relationship, None, content, sent_time, localization)

    assert basic_message.for_relationship == for_relationship
    assert basic_message.content == content
    assert basic_message.sent_time == sent_time
    assert basic_message.localization == localization


@pytest.mark.asyncio
async def test_message():
    context = await Context.create_with_config(await get_test_config())
    basic_message = BasicMessage(for_relationship, None, content, sent_time, localization)
    msg = basic_message.message_msg(context)

    assert msg['@type'] == '{}/{}/{}/{}'.format(
        COMMUNITY_MSG_QUALIFIER,
        BasicMessage.MSG_FAMILY,
        BasicMessage.MSG_FAMILY_VERSION,
        BasicMessage.BASIC_MESSAGE
    )
    assert msg['@id'] is not None
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['content'] == content
    assert msg['sent_time'] == sent_time
    assert msg['~l10n']['locale'] == localization

    await cleanup(context)



# import pytest
#
# from test.test_utils import get_test_config, cleanup
# from verity_sdk.protocols.v1_0.QuestionAnswer import QuestionAnswer
# from verity_sdk.utils import COMMUNITY_MSG_QUALIFIER
# from verity_sdk.utils.Context import Context
#
# for_relationship = 'some_did'
# question_text = 'Are you trying to login to acme.com?'
# question_detail = 'IP Address: 56.24.11.126'
# valid_responses = ['Yes', 'No, that\'s not me!']
# signature_required = True
#
#
# def test_init():
#     question_answer = QuestionAnswer(for_relationship, None, question_text, question_detail, valid_responses,
#                                      signature_required)
#
#     assert question_answer.for_relationship == for_relationship
#     assert question_answer.question == question_text
#     assert question_answer.descr == question_detail
#     assert question_answer.valid_responses == valid_responses
#     assert question_answer.signature_required == signature_required
#
#
# @pytest.mark.asyncio
# async def test_ask():
#     context = await Context.create_with_config(await get_test_config())
#     question_answer = QuestionAnswer(for_relationship, None, question_text, question_detail, valid_responses,
#                                      signature_required)
#     msg = question_answer.ask_msg(context)
#
#     assert msg['@type'] == '{}/{}/{}/{}'.format(
#         COMMUNITY_MSG_QUALIFIER,
#         QuestionAnswer.MSG_FAMILY,
#         QuestionAnswer.MSG_FAMILY_VERSION,
#         QuestionAnswer.ASK_QUESTION
#     )
#     assert msg['@id'] is not None
#     assert msg['~thread'] is not None
#     assert msg['~thread']['thid'] is not None
#     assert msg['~for_relationship'] == for_relationship
#     assert msg['text'] == question_text
#     assert msg['detail'] == question_detail
#     assert msg['valid_responses'] == valid_responses
#     assert msg['signature_required'] == signature_required
#
#     await cleanup(context)
#
#
# @pytest.mark.asyncio
# async def test_status():
#     context = await Context.create_with_config(await get_test_config())
#     question_answer = QuestionAnswer(for_relationship, None, question_text, question_detail, valid_responses,
#                                      signature_required)
#     msg = question_answer.status_msg(context)
#
#     assert msg['@type'] == '{}/{}/{}/{}'.format(
#         COMMUNITY_MSG_QUALIFIER,
#         QuestionAnswer.MSG_FAMILY,
#         QuestionAnswer.MSG_FAMILY_VERSION,
#         QuestionAnswer.GET_STATUS
#     )
#     assert msg['@id'] is not None
#     assert msg['~for_relationship'] == for_relationship
#     assert msg['~thread'] is not None
#     assert msg['~thread']['thid'] is not None
#
#     await cleanup(context)



import pytest

from test.test_utils import get_test_config, cleanup
from verity_sdk.protocols.v0_6.UpdateEndpoint import UpdateEndpoint
from verity_sdk.utils import EVERNYM_MSG_QUALIFIER
from verity_sdk.utils.Context import Context


@pytest.mark.asyncio
async def test_update_endpoint():
    context = await Context.create_with_config(await get_test_config())
    update_endpoint = UpdateEndpoint()
    msg = update_endpoint.update_endpoint_msg(context)

    assert msg['@type'] == '{}/configs/0.6/UPDATE_COM_METHOD'.format(EVERNYM_MSG_QUALIFIER)
    assert msg['@id'] is not None
    assert msg['comMethod']['id'] == 'webhook'
    assert msg['comMethod']['type'] == 2
    assert msg['comMethod']['value'] == context.endpoint_url
    assert msg['comMethod']['packaging']['pkgType'] == '1.0'
    assert msg['comMethod']['packaging']['recipientKeys'] == [context.sdk_verkey]

    await cleanup(context)


@pytest.mark.asyncio
async def test_update_endpoint_with_oauth2_v1_authentication():
    context = await Context.create_with_config(await get_test_config())
    update_endpoint = UpdateEndpoint(
        {
            'type': 'OAuth2',
            'version': 'v1',
            'data': {
                'url': 'https://auth.url/token',
                'grant_type': 'client_credentials',
                'client_id': 'ajeyKDizsDkwYeEmRmHU78gf7W3VIEoA',
                'client_secret': 'aaGxxcGi6kb6AxIe'
            }
        }
    )
    msg = update_endpoint.update_endpoint_msg(context)

    assert msg['@type'] == '{}/configs/0.6/UPDATE_COM_METHOD'.format(EVERNYM_MSG_QUALIFIER)
    assert msg['@id'] is not None
    assert msg['comMethod']['id'] == 'webhook'
    assert msg['comMethod']['type'] == 2
    assert msg['comMethod']['value'] == context.endpoint_url
    assert msg['comMethod']['packaging']['pkgType'] == '1.0'
    assert msg['comMethod']['packaging']['recipientKeys'] == [context.sdk_verkey]

    # authentication
    assert msg['comMethod']['authentication']['type'] == 'OAuth2'
    assert msg['comMethod']['authentication']['version'] == 'v1'
    assert msg['comMethod']['authentication']['data']['url'] == 'https://auth.url/token'
    assert msg['comMethod']['authentication']['data']['grant_type'] == 'client_credentials'
    assert msg['comMethod']['authentication']['data']['client_id'] == 'ajeyKDizsDkwYeEmRmHU78gf7W3VIEoA'
    assert msg['comMethod']['authentication']['data']['client_secret'] == 'aaGxxcGi6kb6AxIe'

    await cleanup(context)
